import QtQuick 2.0
import Ubuntu.Components 0.1
import org.kde.konsole 0.1
import Ubuntu.Unity.Action 1.0 as UnityActions
import "config"
import "extraPanel"

Item {
    id: mparent
    height: Qt.inputMethod.visible ? parent.height - Qt.inputMethod.keyboardRectangle.height : parent.height
    width: parent.width

    function hideFocus(){
        kterm.focus = false;
    }

    onWidthChanged: {
        hideFocus();
    }

    onHeightChanged: {
        screenRow.vheight = height;
    }

    //property int  headerHeight:  units.gu(10);

    function showExtraPanel(i){
        switch (i){
            case 1:
                kbCtrl.visible = true;
                kbFn.visible   = false;
                kbScrl.visible = false;
                break;
            case 2:
                kbCtrl.visible = false;
                kbFn.visible   = true;
                kbScrl.visible = false;
                break;
            case 3:
                kbCtrl.visible = false;
                kbScrl.visible = true;
                kbFn.visible   = false;
                break;
            default:
                kbCtrl.visible = false;
                kbFn.visible   = false;
                kbScrl.visible = false;
        }
    }

    Component.onCompleted: {
        pgConf.colorSchemes = kterm.availableColorSchemes();

        mview.actionManager.addLocalContext(actionContext);

        // Change directory to $HOME
        ksession.sendText("cd $HOME;clear;\n")
    }

    Connections {
        target: pgConf
        onConfigChanged: {
            kterm.font.pointSize = jsConf.getFontSize();
            kterm.colorScheme = jsConf.getColorScheme();
        }
    }

    Connections {
        target: Qt.application
        onActiveChanged: hideFocus();
    }

    JsConfig{id: jsConf }


    Column {
        id: mcol
        anchors.fill: parent
        spacing: units.gu(1)
        anchors.topMargin: {
            if (!isExtraPanelVisible() === false) {
                units.gu(1)
            }
        }

        // hack for proper height
        onHeightChanged: {
            //screenRow.vheight = height - screenRow.shift
            console.log("Column.height = " + height)
        }

        //Component.onCompleted: showExtraPanel()

        CtrlKeys {
            id: kbCtrl
            objectName: "kbCtrl"
            visible: false
            width: parent.width
            anchors{
                left: parent.left
                leftMargin: units.gu(3)
            }
            onHeightChanged: {
                console.log("kbCtrl.height = " + height)
            }
            onKeyPressed: ksession.sendKey(1, key, mod)
            onVisibleChanged: visible ? screenRow.shift += (height + mcol.spacing) : screenRow.shift = 0
        }

        FnKeys {
            id: kbFn
            objectName: "kbFn"
            visible: false
            width: parent.width
            anchors{
                left: parent.left
                leftMargin: units.gu(0.5)
            }
            onHeightChanged: {
                console.log("kbFn.height = " + height)
            }
            onKeyPressed: ksession.sendKey(1, key, mod)

            onVisibleChanged: visible ? screenRow.shift += (height + mcol.spacing) : screenRow.shift = 0
        }

        ScrlKeys {
            id: kbScrl
            objectName: "kbScrl"
            visible: false
            width: parent.width
            anchors{
                left: parent.left
                leftMargin: units.gu(1)
            }
            onHeightChanged: {
                console.log("kbScrl.height = " + height)
            }
            onKeyPressed: ksession.sendKey(1, key, mod)
            onVisibleChanged: visible ? screenRow.shift += (height + mcol.spacing) : screenRow.shift = 0
        }

        Row {
            id: screenRow
            height: vheight - shift
            width: parent.width

            property real shift: 0
            property real vheight: parent.height

            //...

            Item {
                anchors.fill: parent

                CircleMenu {
                    id: cmenu
                    objectName: "cmenu"
                    z: 99
                    visible: false
                    anchors.centerIn: parent
                    circleButtonLable: "CTRL +"
                    sideButtons: ["A", "C", "D", "S", "X", "Z"]


                    onCircleButtonClicked: console.log("Circle button clicked")
                    onSideButtonClicked: {
                        switch (index) {
                            case 0:
                                // CTRL+A
                                ksession.sendKey(1, Qt.Key_A, Qt.ControlModifier);
                                break;
                            case 1:
                                // CTRL+C
                                ksession.sendKey(1, Qt.Key_C, Qt.ControlModifier);
                                break
                            case 2:
                                // CTRL+D
                                ksession.sendKey(1, Qt.Key_D, Qt.ControlModifier);
                                break
                            case 3:
                                // CTRL+S
                                ksession.sendKey(1, Qt.Key_S, Qt.ControlModifier);
                                break
                            case 4:
                                // CTRL+X
                                ksession.sendKey(1, Qt.Key_X, Qt.ControlModifier);
                                break
                            case 5:
                                // CTRL+Z
                                ksession.sendKey(1, Qt.Key_Z, Qt.ControlModifier);
                                break
                        }
                    }
                }

                KTerminal {
                    id: kterm
                    objectName: "kterm"

                    //////////////////////////////////////
                    /// FONT SIZE = 24 (FINE FOR GNEXUS)
                    /// FONT SIZE = 12 (FINE FOR DESKTOP)
                    //////////////////////////////////////
                    font.family: "Ubuntu Mono"
                    font.pointSize: jsConf.getFontSize() //units.gu(1.5)

                    colorScheme: jsConf.getColorScheme() //"DarkPastels"
                    width:  parent.width
                    height: parent.height

                    session: KSession {
                        id: ksession
                        objectName: "ksession"
                        kbScheme: "linux"

                        onFinished: {
                            Qt.quit()
                        }
                    }
                }

                MouseArea {
                    anchors.fill: parent
                    enabled: true
                    //preventStealing: false

                    property int  cPosX: 0
                    property int  cPosY: 0

                    property int  prevPosX:  0
                    property int  prevPosY:  0

                    property bool swiped: false

                    onPressed: {
                        prevPosX = cPosX = mouse.x;
                        prevPosY = cPosY = mouse.y;

                        kterm.click(mouse.x,mouse.y);

                        if (cmenu.visible) cmenu.visible = false;
                    }

                    onReleased: {
                        swiped = false;
                    }

                    onDoubleClicked: {
                        ksession.sendKey(1, Qt.Key_Tab, Qt.NoModifier)
                    }

                    onPressAndHold: {
                        if (!swiped) cmenu.visible = true;
                    }

                    onPositionChanged: {
                        var dltX   = mouse.x - prevPosX;
                        var dltY   = mouse.y - prevPosY;
                        var s1 = 50;
                        var s2 = 30;

                        if (Math.abs(dltX) > s1 || Math.abs(dltY) > s1) {

                            swiped = true;

                            if (dltX > s2 && Math.abs(dltY) < s2) {
                                // swipe right
                                ksession.sendKey(1, Qt.Key_Right, Qt.NoModifier)
                            } else if (dltX < -s2 && Math.abs(dltY) < s2) {
                                // swipe left
                                ksession.sendKey(1, Qt.Key_Left, Qt.NoModifier)
                            } else if (Math.abs(dltX) < s2 && dltY > s2) {
                                // swipe down
                                ksession.sendKey(1, Qt.Key_Down, Qt.NoModifier)
                            } else if (Math.abs(dltX) < s2 && dltY < s2) {
                                // swipe up
                                ksession.sendKey(1, Qt.Key_Up, Qt.NoModifier)
                            }

                            prevPosX = mouse.x;
                            prevPosY = mouse.y;
                        }

                    }
                }

            }

            //...

        }

        function isExtraPanelVisible() {
            if (kbCtrl.visible) {
                return true;
            } else if (kbFn.visible) {
                return true;
            } else if (kbScrl.visible) {
                return true;
            }
            else {
                return false;
            }
        }

    }

    UnityActions.ActionContext {
        id: actionContext
        active: mparent.visible
        actions: [
            UnityActions.Action {
                text: i18n.tr("Upload Crash Reports")
                keywords: i18n.tr("Whoopsie;Apport;Debugging;Bugs")
                onTriggered: ksession.sendText("\x03\n/usr/share/apport/whoopsie-upload-all\n")
            },
            UnityActions.Action {
                text: i18n.tr("Make Image Writable")
                keywords: i18n.tr("System Image;Install Deb")
                onTriggered: ksession.sendText("\x03\nsudo touch /userdata/.writable_image\n")
            },
            UnityActions.Action {
                text: i18n.tr("List Running Applications")
                keywords: i18n.tr("Apps")
                onTriggered: ksession.sendText("\x03\nupstart-app-list\n")
            },
            UnityActions.Action {
                text: i18n.tr("Installed Click Packages")
                keywords: i18n.tr("Packages;Applications;Utilities")
                onTriggered: ksession.sendText("\x03\nclick list\n")
            },
            UnityActions.Action {
                text: i18n.tr("System Image Information")
                keywords: i18n.tr("Image;Version;Channel")
                onTriggered: ksession.sendText("\x03\nsudo system-image-cli -i\n")
            },
            UnityActions.Action {
                text: i18n.tr("System Kernel Version")
                onTriggered: ksession.sendText("\x03\nuname -a\n")
            },
            UnityActions.Action {
                text: i18n.tr("Upstart Job Status")
                keywords: i18n.tr("Init;Session;User Job")
                onTriggered: ksession.sendText("\x03\ninitctl list | sort\n")
            },
            UnityActions.Action {
                text: i18n.tr("Hack into the NSA")
                keywords: i18n.tr("Government;Secret;Cracking")
                onTriggered: ksession.sendText("\x03\nclear;echo -n \"That's not very nice, now tracing your location .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";sleep 1;echo -n \" .\";echo \"Trace failed\";echo \"You get away this time, but don't try again.\"\n")
            },
            UnityActions.Action {
                text: i18n.tr("Watch Running Processes")
                keywords: i18n.tr("Top;Memory;Usage")
                onTriggered: ksession.sendText("\x03\ntop\n")
            },
            UnityActions.Action {
                text: i18n.tr("Networking Status")
                keywords: i18n.tr("Wireless;Ethernet;Access Points")
                onTriggered: ksession.sendText("\x03\nnm-tool\n")
            },
            UnityActions.Action {
                text: i18n.tr("Watch Star Wars Episode IV")
                keywords: i18n.tr("Death Star;Skywalker;Leia;Darth Vader")
                onTriggered: ksession.sendText("\x03\nnetcat -v towel.blinkenlights.nl 23\n")
            }
        ]
    }
}
