import QtQuick 2.0
import Ubuntu.Components 0.1
import QtGraphicalEffects 1.0

Item {
    id: mparent
    height: units.gu(30)
    width: units.gu(30)

    property variant sideButtons: [ "1", "2", "3", "4" ]
    property alias circleButtonLable: lbCircle.text

    signal sideButtonClicked( int index)
    signal circleButtonClicked
    //signal outsideClicked

    //onSideButtonClicked: console.log("Clicked segment " + (index+1));

    Repeater {
      model: sideButtons.length/2
      delegate: Rectangle {
          width: btnTorus.radius*2
          height: units.gu(0.25)
          color: "grey"
          antialiasing: true
          x: 0
          y: btnTorus.radius - height/2
          rotation: (index * marea.anglePerSegment + marea.anglePerSegment/2) * 180 / Math.PI
          z: 89
      }
    }

    Repeater {
      id: btnSider
      model: sideButtons.length
      delegate: Rectangle {
          width: 30
          height: 30
          color: "transparent"
          x: parent.width / 2 + ( btnTorus.radius - units.gu(4)) *
               Math.cos(index * marea.anglePerSegment ) - width/2;
          y: parent.height / 2 - ( btnTorus.radius - units.gu(4)) *
               Math.sin(index * marea.anglePerSegment ) - height/2;
          z: 90

          Label {
              //color: "black"
              fontSize: "large"
              anchors.centerIn: parent
              text: sideButtons[index]
          }

      }
    }

    Rectangle {
        id: btnTorus
        anchors.fill: parent
        radius: width/2
        color: "lightgrey"
        antialiasing: true
    }

    ConicalGradient {
        id: effect
        anchors.fill: parent
        visible: selSegment != -1
        source: btnTorus
        rotation: {
            var angel = marea.anglePerSegment * ( 180 / Math.PI);
            return  ((sideButtons.length - selSegment) * angel  +
                      (90 + angel/2)) - angel;
        }

        property real  posPerSegment: 1/sideButtons.length
        property int   selSegment: -1
        property color color: "#85838388"

        gradient: Gradient {
            GradientStop { position: 0; color: "transparent" }
            GradientStop {
                position: 0.001
                color: effect.color
            }
            GradientStop {
                position: effect.posPerSegment - 0.001
                color: effect.color
            }
            GradientStop { position: effect.posPerSegment; color: "transparent" }
        }

    }

    Rectangle {
        color: "grey"
        antialiasing: true
        z: 91
        anchors{
            verticalCenter: parent.verticalCenter
            horizontalCenter: parent.horizontalCenter
        }

        width: parent.width/2.5 + units.gu(0.5)
        height: width
        radius: width/2
    }

    Rectangle {
        id: shdwCircle
        color: "#85838388"
        antialiasing: true
        visible: false
        z: 93
        anchors{
            verticalCenter: parent.verticalCenter
            horizontalCenter: parent.horizontalCenter
        }

        width: parent.width/2.5
        height: width
        radius: width/2
    }

    Rectangle {
        id: btnCircle
        color: "lightgrey"
        antialiasing: true
        z: 92
        anchors{
            verticalCenter: parent.verticalCenter
            horizontalCenter: parent.horizontalCenter
        }

        width: parent.width/2.5
        height: width
        radius: width/2

        Label {
            id: lbCircle
            //color: "black"
            fontSize: "large"
            anchors.centerIn: parent
            text: "Central"
        }
    }

    MouseArea {
        id: marea
        z: 99
        preventStealing: true
        anchors.fill: parent
        enabled: mparent.visible

        property real radius: btnTorus.width/2
        property real inner_radius: btnCircle.radius
        property real anglePerSegment: (2 * Math.PI) / sideButtons.length

        onPressed: {

            var x = mouse.x - radius;
            var y = mouse.y - radius;
            var a = Math.atan2(-y, x) < 0 ? 2*Math.PI + Math.atan2(-y, x) : Math.atan2(-y, x);
            var r = Math.sqrt( Math.pow(x, 2) + Math.pow(y, 2) );

            if ( r < radius ) {
                if ( r > inner_radius ) {
                    for (var i = 0; i < sideButtons.length; i++){

                        if ( ( (i * anglePerSegment - marea.anglePerSegment/2) < a &&
                               ((i+1) * anglePerSegment - marea.anglePerSegment/2) > a ) ||
                               (i == 0 && (sideButtons.length * anglePerSegment  - marea.anglePerSegment/2) < a &&
                               ((( sideButtons.length + 1 ) * anglePerSegment  - marea.anglePerSegment/2) > a )))
                        {
                            effect.selSegment = i;
                            sideButtonClicked(i);
                        }
                    }
                    //console.log("Clicked inside torus")
                } else {
                    shdwCircle.visible = true;
                    circleButtonClicked();
                    //console.log("Clicked inside inner circle")
                }

            }
            else {
                //outsideClicked();
                mparent.visible = false;
            }

        }

        onReleased: {
            if (effect.selSegment != -1) effect.selSegment = -1;
            if (shdwCircle.visible) shdwCircle.visible = false;
            mparent.visible = false;
        }
    }
}
