# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Terminal app autopilot emulators."""

from time import sleep
from ubuntuuitoolkit import emulators as toolkit_emulators


class MainView(toolkit_emulators.MainView):

    """Terminal MainView Autopilot emulator."""

    def get_main_view(self):
        return self.wait_select_single("QQuickView")

    def get_slider_item(self, slider):
        return self.wait_select_single("Slider", objectName=slider)

    def get_kterm(self):
        return self.wait_select_single("KTerminalDisplay", objectName="kterm")

    def get_control_panel(self):
        return self.wait_select_single("CtrlKeys", objectName="kbCtrl")

    def get_function_panel(self):
        return self.wait_select_single("FnKeys", objectName="kbFn")

    def get_scroll_panel(self):
        return self.wait_select_single("ScrlKeys", objectName="kbScrl")

    def get_terminal_page(self):
        return self.wait_select_single("Terminal", objectName="pgTerm")

    def get_circle_menu(self):
        return self.wait_select_single("CircleMenu", objectName="cmenu")

    def get_panel_actions_popover(self):
        """Return the ActionSelectionPopover emulator of the popup action."""
        return self.wait_select_single(ActionSelectionPopover,
                                       objectName='panelpopover')

    def drag_horizontal_slider(self, slider, pos):
        """Drag slider until value is set"""
        slItem = self.get_slider_item(slider)

        slRect = slItem.select_single("SliderStyle")

        slideMin = int(slItem.minimumValue)
        slideMax = int(slItem.maximumValue)

        if pos > slideMax:
            raise ValueError("Pos cannot be greater than" + str(slideMax))

        if pos < slideMin:
            raise ValueError("Pos cannot be less than" + str(slideMin))

        x, y, w, h = slRect.globalRect
        # calculate the approximate slide step width
        # we take half of the theoretical step value just to make
        # sure we don't miss any values
        step = w / ((slideMax - slideMin) * 2)
        sx = x + step
        sy = y + h / 2
        loop = 1

        # set the slider to minimum value and loop
        self.pointing_device.move(sx, sy)
        self.pointing_device.click()

        # drag through the slider until the pos matches our desired value
        # in case of bad sliding, loop will also timeout
        while round(slItem.value) != pos and slItem.value < slideMax \
                and loop <= (slideMax * 2):
                valuePos = int(sx + step)
                self.pointing_device.drag(sx, sy, valuePos, sy)
                sx = valuePos
                sleep(0.3)
                loop = loop + 1


class ActionSelectionPopover(toolkit_emulators.UbuntuUIToolkitEmulatorBase):

    """ActionSelectionPopover Autopilot emulator."""
    # TODO Move this to the ubuntu-ui-toolkit. Reported on
    # https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1205205
    # --elopio - 2013-07-25

    def __init__(self, *args):
        super(ActionSelectionPopover, self).__init__(*args)
        self.pointing_device = toolkit_emulators.get_pointing_device()

    def click_button(self, text):
        """Click a button on the popover.

        XXX We are receiving the text because there's no way to set the
        objectName on the action. This is reported at
        https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1205144
        --elopio - 2013-07-25

        :parameter text: The text of the button.

        """
        button = self._get_button(text)
        if button is None:
            raise ValueError(
                'Button with text "{0}" not found.'.format(text))
        self.pointing_device.click_object(button)

    def _get_button(self, text):
        buttons = self.select_many('Empty')
        for button in buttons:
            if button.text == text:
                return button
